/*
 * Copyright 2004-2006 H2 Group. Licensed under the H2 License, Version 1.0 (http://h2database.com/html/license.html).
 * Initial Developer: H2 Group
 */
package org.h2.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.h2.message.Message;
import org.h2.store.FileLister;
import org.h2.util.FileUtils;
import org.h2.util.IOUtils;

/*
 * Backs up a H2 database by creating a .zip file from the database files.
 * 
 * @author Thomas
 */
public class Backup {

    private void showUsage() {
        System.out.println("java "+getClass().getName()
                + " [-file <filename>] [-dir <dir>] [-db <database>] [-quiet]");
    }
    
    /**
     * The command line interface for this tool.
     * The options must be split into strings like this: "-db", "test",... 
     * The following options are supported:
     * <ul>
     * <li>-help or -? (print the list of options)
     * </li><li>-file filename (the default is backup.zip)
     * </li><li>-dir directory (the default is the current directory)
     * </li><li>-db databaseName (all databases if no name is specified)
     * </li><li>-quiet does not print progress information
     * </li></ul>
     * 
     * @param args the command line arguments
     * @throws SQLException
     */    
    public static void main(String[] args) throws SQLException {
        new Backup().run(args);
    }
    
    private void run(String[] args) throws SQLException {
        String zipFileName = "backup.zip";
        String dir = ".";
        String db = null;
        boolean quiet = false;
        for(int i=0; args != null && i<args.length; i++) {
            if(args[i].equals("-dir")) {
                dir = args[++i];
            } else if(args[i].equals("-db")) {
                db = args[++i];
            } else if(args[i].equals("-quiet")) {
                quiet = true;
            } else {
                showUsage();
                return;
            }
        }
        Backup.execute(zipFileName, dir, db, quiet);
    }

    /**
     * Backs up database files.
     * 
     * @param zipFileName the name of the backup file
     * @param directory the directory name
     * @param db the database name (null for all databases)
     * @param quiet don't print progress information
     * @throws SQLException
     */    
    public static void execute(String zipFileName, String directory, String db, boolean quiet) throws SQLException {
        ArrayList list = FileLister.getDatabaseFiles(directory, db, true);
        if (list.size() == 0 && !quiet) {
            System.out.println("No database files found");
            return;
        }
        File file = new File(zipFileName);
        if(file.exists()) {
            file.delete();
        }
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(file);
            ZipOutputStream zipOut = new ZipOutputStream(out);
            for(int i=0; i<list.size(); i++) {
                String fileName = (String) list.get(i);
                ZipEntry entry = new ZipEntry(FileUtils.getFileName(fileName));
                zipOut.putNextEntry(entry);
                FileInputStream in = null;
                try {
                    in = new FileInputStream(fileName);
                    IOUtils.copyAndCloseInput(in, zipOut);
                } finally {
                    IOUtils.closeSilently(in);
                }
                zipOut.closeEntry();
                if (!quiet) {
                    System.out.println("processed: " + fileName);
                }                
            }
            zipOut.closeEntry();
            zipOut.close();
        } catch(IOException e) {
            throw Message.convert(e);
        } finally {
            IOUtils.closeSilently(out);
        }
    }

}
